#____________________________________________________________________________________
#
#    contentinfo (see README.txt for more information)
#    Copyright (C) 2002 Christoph Becker-Freyseng
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#    webmaster@beyond-thoughts.com
#    http://www.beyond-thoughts.com
#    http://www.mmc-startup.com
#____________________________________________________________________________________
#




__doc__=""" This module will give you extra-information for files and other data.
    It will use mimetypes, analyse your data/file and return the type and extra-info """
__version__='0.1.0'


import _oneTimes
from CIExceptions import *
import CIContentTypes

import os
import string
import imp
import mimetypes

try:
    import OFS.content_types	# if we are in Zope use this module
    def guess_mime(fname, data):
	noneMark ='just nothing UAAAA'
	mt, enc = OFS.content_types.guess_content_type(fname, data, noneMark)
	if mt is noneMark:
	    mt = None
	    enc = None
	return (mt, enc)
except:
    def guess_mime(fname, data):
	return mimetypes.guess_type(fname)


_moreMimeTypes= (
                    ('.png', 'image/png'),
                    ('.ra', 'audio/vnd.rn-realaudio'),
                    ('.ram', 'application/x-pn-realaudio'),
		    ('.swf', 'application/x-shockwave-flash'),
		    ('.mp3', 'audio/mpeg'),
		    ('.mov', 'video/quicktime'),
                )
for suffixR, mTypeR in _moreMimeTypes:
    if not mimetypes.types_map.has_key(suffixR):
	mimetypes.types_map[suffixR]= mTypeR






if _oneTimes.isFirstTime():
    analysersDict= {}
    
    def registerFunc(mimetype, analyseFunc, priority=0, analysersDict=analysersDict):
	listD= []
	oldListD= analysersDict.get(mimetype, [])
	for eleR in oldListD:
	    if eleR[1] <= priority:
		listD.append((analyseFunc, priority))
	    listD.append(eleR)
	if len(listD)==len(oldListD):
	    listD.append((analyseFunc, priority))
	analysersDict[mimetype]= listD
    
    for fileR in os.listdir(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'contentAnalysers')):
	if fileR.endswith('.py') and (not fileR.endswith('__init__.py')):
	    fullFnD= os.path.join(os.path.dirname(os.path.abspath(__file__)), 'contentAnalysers', fileR)
	    modNameD= __name__+'.contentAnalysers.'+fileR[:-3]
	    fileD= open(fullFnD, 'r')
	    modNameD=''		# Otherwise we get trouble ...
	    try:
		anaD= imp.load_source(modNameD, fullFnD, fileD)
	    finally:
		fileD.close()
	    anaD.initAnalysers(registerFunc)










def getContentinfo(file, hard=1, returnData=0, fname=None):
    if fname is None:
	mimetypeStd= None
	extStd= None
    else:
	if string.find(fname, ':')==1:	# Windows-Filename
	    fname= string.replace(fname,'\\','/')
	mimetypeStd, encD= guess_mime(fname, datahead)
	ppD= string.rfind(fname, '.')
	if ppD>0:
	    extStd= fname[ppD:]
	else:
	    extStd= None
    if type(file) is type(''):	# this is just data
        data= file
	datahead= data
	fullsize= len(data)
	mimetype= mimetypeStd
	extD= extStd
    else:
	try:
	    if returnData:
		file.seek(0)
	        data = file.read()
		datahead = data
	    else:
		data = None
	        datahead = None
	    file.seek(0, 2)
	    fullsize = file.tell()
	    file.seek(0)
	    fname = getattr(file, 'name', '')
	    fname = getattr(file, 'filename', fname)
	    if string.find(fname, ':')==1:	# Windows-Filename
    		fname=string.replace(fname,'\\','/')
	    headers=getattr(file, 'headers', None)
	    mimetype, encD=guess_mime(fname, data)
	    if mimetype is None:  # we can't guess it... maybe the browser
		if headers and headers.has_key('content-type'):
		    mimetype=headers['content-type']
	    if mimetype is None:  # still None -> use the deafult/spare value
		mimetype= mimetypeStd
	    ppD = string.rfind(fname, '.')
	    if ppD>0:
		extD = fname[ppD:]
	    else:
		extD = extStd
	except:		# Strange file ...
	    data = file.read()
	    datahead = data
	    fullsize = len(data)
	    mimetype = mimetypeStd
	    extD = extStd
    info = None
    if data is None: datahead = file.read(64)
    while (info is None):
	try:
	    info= getContentinfo_mime(datahead, fullsize, mimetype, hard)
	except DataheadError:
	    if (data is None) and (len(datahead)<fullsize):
		datahead = datahead + file.read(64)
	    else:
		info= CIContentTypes.FileContent(fullsize)
    if not getattr(info, 'extension', None):
	setattr(info, 'extension', extD)
    if returnData:
	return (data, info)
    else:
	return info

def getContentinfo_fname(datahead, fullsize, fname, hard=1):
    if string.find(fname, ':')==1:	# Windows-Filename
        fname=string.replace(fname,'\\','/')
    mimetype, encD=guess_mime(fname, datahead)
    ppD = string.rfind(fname, '.')
    if ppD>0:
	extD = fname[ppD:]
    else:
        extD = None
    info = getContentinfo_mime(datahead, fullsize, mimetype, hard)
    extD = info[2].get('extension', None)
    if not extD:
	ppD = string.rfind(fname, '.')
	if ppD>0:
	    extD = fname[ppD:]
	else:
	    extD = None
    info[2]['extension'] = extD
    return info

def getContentinfo_mime(datahead, fullsize, mimetype, hard=1):
    analyseFuncList = analysersDict.get(mimetype, [])
    for analyseFuncTupleR in analyseFuncList:
	analyseFunc= analyseFuncTupleR[0]
	try:
	    return analyseFunc(datahead, fullsize)
	except UnknownContentError: pass
	except BrokenContentError: pass
	# So there was an error
	extD = mimetypes.guess_extension(mimetype)
    # So we have no good mimetype till now
    if hard:
	for analyseFuncList in analysersDict.values():
	    for analyseFuncTupleR in analyseFuncList:
		analyseFunc= analyseFuncTupleR[0]
		try:
		    return analyseFunc(datahead, fullsize)
		except UnknownContentError: pass
		except BrokenContentError: pass
    if not mimetype:
	mimetype = 'text/x-unknown-content-type'
	extD = None
    else:
	extD = mimetypes.guess_extension(mimetype)
    return CIContentTypes.FileContent(fullsize, mimetype, extD)



def _mainFunc():
    print 'Use _exec_contentinfo instead'

if __name__ == '__main__':
    _mainFunc()


