#____________________________________________________________________________________
#
#    contentinfo (see README.txt for more information)
#    Copyright (C) 2002 Christoph Becker-Freyseng
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#    webmaster@beyond-thoughts.com
#    http://www.beyond-thoughts.com
#    http://www.mmc-startup.com
#____________________________________________________________________________________
#




__doc__="""this module analyses a file when it's a .swf (Flash/Shockwave)"""
__version__='0.1.0'


import CIContentTypes
import CIExceptions


def getUBits(bitList, bitPos, nr): #unsigned
    valD = 0
    for runD in xrange(bitPos, bitPos+nr):
	valD = valD*2 + bitList[runD]
    return valD

def getSBits(bitList, bitPos, nr): #signed
    valD = 0
    for runD in xrange(bitPos+1, bitPos+nr): #skip the first bit -> it is the sign
        valD = valD*2 + bitList[runD]
    if bitList[bitPos]: valD = -valD
    return valD


def analyseContent(datahead, fullsize):
    """ opens the flash file and checks if it is a valid
	flash file, the size, the version... """
    if len(datahead)<=(8+17):	# see below, why we need at least this size
        raise CIExceptions.DataheadError(tooSmall=1)
    if str(datahead[:3]) == 'FWS':
        version = str(ord(datahead[3]))
    else:
	raise CIExceptions.UnknownContentError()
    size = ord(datahead[4]) + ord(datahead[5])*256 + ord(datahead[6])*256*256 + ord(datahead[7])*256*256*256
    if (size!=fullsize) or (size<21):	# must be at least 21 bytes long
        raise CIExceptions.BrokenContentError()
    # we need to read the next 17 bytes for size etc at maximum
    # because first field is 5 bits and the next four are max 32 bit
    # because 2 pow 5 = 32
    bitList = []
    for runD in xrange(8, 8+17):
        byteD = ord(datahead[runD])
        bitList.append( (byteD&128)/128 )
        bitList.append( (byteD&64)/64 )
        bitList.append( (byteD&32)/32 )
        bitList.append( (byteD&16)/16 )
        bitList.append( (byteD&8)/8 )
        bitList.append( (byteD&4)/4 )
        bitList.append( (byteD&2)/2 )
        bitList.append( (byteD&1)/1 )
    bitPos = 0
    fieldSize = getUBits(bitList, bitPos, 5); bitPos=bitPos+ 5
    x1 = getSBits(bitList, bitPos, fieldSize); bitPos=bitPos+ fieldSize
    x2 = getSBits(bitList, bitPos, fieldSize); bitPos=bitPos+ fieldSize
    y1 = getSBits(bitList, bitPos, fieldSize); bitPos=bitPos+ fieldSize
    y2 = getSBits(bitList, bitPos, fieldSize); bitPos=bitPos+ fieldSize
    return FlashContent(fullsize, version, x1, y1, x2, y2)



class FlashContent(CIContentTypes.FileContent, CIContentTypes.PictureContent, CIContentTypes.CodecContent):
    def __init__(self, fullsize, codec, flash_x1, flash_y1, flash_x2, flash_y2):
        self.mimetype = 'application/x-shockwave-flash'
        self.extension= '.swf'
        self.fullsize= fullsize
        self.codec= codec
	self.flash_x1= flash_x1
	self.x1= int(flash_x1/20)
	self.flash_y1= flash_y1
	self.y1= int(flash_y1/20)
	self.flash_x2= flash_x2
	self.x2= int(flash_x2/20)
	self.flash_y2= flash_y2
	self.y2= int(flash_y2/20)
	self.width= int( (flash_x2 - flash_x1) /20)    # Whaaaaa they store the size in pixel*20
        self.height= int( (flash_y2 - flash_y1) /20)    # Whaaaaa they store the size in pixel*20




def initAnalysers(registerFunc):
    registerFunc('application/x-shockwave-flash', analyseContent)



