#____________________________________________________________________________________
#
#    contentinfo (see README.txt for more information)
#    Copyright (C) 2002 Christoph Becker-Freyseng
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#    webmaster@beyond-thoughts.com
#    http://www.beyond-thoughts.com
#    http://www.mmc-startup.com
#____________________________________________________________________________________
#




__doc__="""this module analyses a file when it's a .png
See http://www.w3.org/TR/PNG-Structure.html
"""
__version__='0.1.0'


import CIContentTypes
import CIExceptions

import struct
from cStringIO import StringIO


def analyseContent(datahead, fullsize):
    if len(datahead)<(8+4+4+13+4):	# ID, length, header-type, data, crc
        raise CIExceptions.DataheadError(tooSmall=1)
    dataStream= StringIO(datahead)
    if dataStream.read(8) != '\211PNG\r\n\032\n':
	raise CIExceptions.UnknownContentError()
    if fullsize<33:
	raise CIExceptions.BrokenContentError()
    # Next we analys the IHDR  Image header -- The IHDR chunk must appear FIRST
    hLength= struct.unpack('>I', dataStream.read(4))[0]
    print "png"
    print hLength
    if dataStream.read(4) != 'IHDR': # if the first chunk-type isn't IHDR the file is broken
	raise CIExceptions.BrokenContentError()
    width= struct.unpack('>i', dataStream.read(4))[0]
    height= struct.unpack('>i', dataStream.read(4))[0]
    bitDepth= struct.unpack('>b', dataStream.read(1))[0]
    colorType= struct.unpack('>b', dataStream.read(1))[0]
    compression= struct.unpack('>b', dataStream.read(1))[0]
    filterMethod= struct.unpack('>b', dataStream.read(1))[0]
    interlaceMethod= struct.unpack('>b', dataStream.read(1))[0]
    crc= dataStream.read(4)	# XXX no crc check yet implemented
    return PngContent(fullsize, width, height, bitDepth, colorType, compression, filterMethod, interlaceMethod)



class PngContent(CIContentTypes.FileContent, CIContentTypes.PictureContent):	# The class for the content of a png-file
    
    colorTypeDescr= {
			0: 'Each pixel is a grayscale sample ',
			2: 'Each pixel is an R,G,B triple ',
			3: 'Each pixel is a palette index (see PLTE chunk for interpretation) ',
			4: 'Each pixel is a grayscale sample, followed by an alpha sample ',
			6: 'Each pixel is an R,G,B triple, followed by an alpha sample ',
		    }
    interlaceMethodDescr= {
			    0: 'no interlace',
			    1: 'Adam7 interlace',
			}
    
    def __init__(self, fullsize, width, height, bitDepth, colorType, compression, filterMethod, interlaceMethod):
	self.mimetype = 'image/png'
        self.extension= '.png'
        self.fullsize= fullsize
	self.width= width
	self.height= height
	self.bitDepth= bitDepth
	self.colorType= colorType
	self.compression= compression
	self.filterMethod= filterMethod
	self.interlaceMethod= interlaceMethod

    def __str__(self):
        strD= ''
        strD+= 'ContentType-Class:   '+str(type(self))+' :'
        strD+= '\n   mimetype = '+str(self.mimetype)
	strD+= '\n   extension = '+str(self.extension)
	strD+= '\n   fullsize = '+str(self.fullsize)
	strD+= '\n   width = '+str(self.width)
	strD+= '\n   height = '+str(self.height)
	strD+= '\n   Colors : '+str(self.colorTypeDescr.get(self.colorType, '?? '))
	strD+= 'with a depth of '+str(self.bitDepth)+ ' bits.'
	strD+= '\n   compression = '+str(self.compression)
	strD+= '\n   filterMethod = '+str(self.filterMethod)
	strD+= '\n   interlaceMethod : '+str(self.interlaceMethodDescr.get(self.interlaceMethod, '?? '))
        return strD





def initAnalysers(registerFunc):
    registerFunc('image/png', analyseContent)

